#ifndef COMM_H
#define COMM_H

#include <stdio.h>
#include "mpi.h"
#include "Graph.h"
#include "StackGraph.h"
#include "StackGraphItem.h"

class Comm {
  FILE *fout;

  static Comm *instance;
  int rank;
  int processes;
  int donor;
  int state;
  bool inquiry;
  bool requestActive;
  bool terminated;
  double startTime;
  double endTime;
  double totalTime;
  int requestTotal;
  int requestSucc;
  MPI_Status status;
  static char ONE;
  static char ZERO;

  Comm(int argc, char *argv[]);
public:
  static const int GRAPH = 1;
  static const int STACK = 2;
  static const int SOLUTION = 3;
  static const int JOB_REQ = 4;
  static const int TOKEN = 5;
  static const int TERMINATE = 6;
  static const int CONFIG = 7;

  static const int WAITING = 0;
  static const int BUSY = 1;

  static Comm *getInstance(int argc, char *argv[]) {if(instance == NULL) instance = new Comm(argc, argv); return instance;}
  static Comm *getInstance() {return instance;}
  ~Comm() {MPI_Finalize(); if(fout) fclose(fout);}
  int getRank() {return rank;}
  int getProcessesCount() {return processes;}
  int getMessageType() {return status.MPI_TAG;}
  int getMessageSource() {return status.MPI_SOURCE;}
  void setState(int pState) {state = pState;}
  int getState() {return state;}
  int ready() {int flag; MPI_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &flag, &status); return flag;}
  bool working() {return !terminated;}
  void sendGraph(Graph *graph);
  void sendStack(int process, StackGraphItem **stack, int count, int edges);
  void sendSolution(int solution);
  Graph *receiveGraph();
  void receiveStack(StackGraph *stack);
  int receiveSolution();
  void requestJob();
  void ackJob();
  void receiveJob(StackGraph *stack);
  void sendToken(int pProcess, int pToken);
  int receiveToken();
  void sendTerminate(int pSolProcess);
  void acceptTerminate(bool *pSolution, int pEdges);
  void closeInquiry() {inquiry = false;}
  bool inquiryRunning() {return inquiry;}
  void sendConfig(int pDest, bool *pSolution, int pEdges);
  void receiveConfig(bool **pSolution);
  void start();
  void stop();
  double getTotalTime() {return totalTime;}
};

#endif /* COMM_H */

